@@ -24,6 +24,10 @@ class CompilationSetting(Enum):
     INSTANT = "instant"
     ON_DEMAND = "on_demand"
 
+def _wrap_arg(value: Any) -> Dict[str, Any]:
+    """Wrap an argument value in the $type/value structure"""
+    return {"$type": "Constant", "value": value}
+
 # Helper functions for building filter expressions
 class Filter:
     """Helper class for building filter expressions for get_nodes()"""
@@ -172,7 +176,7 @@ class OatClient:
     def _resolve_id(self, given_id: str) -> str:
         """
             Three possible ways to assign the correct id:
-            
+
                 1. The ID is from the DB, in which case it is used directly. This means not in _id_map nor in buffer.
                 2. The ID is from a previous function call, in which case it is mapped via _id_map.
                 3. The ID is yet to be used, is therefore in the buffer and is wrapped in a get_result call.
@@ -180,7 +184,7 @@ class OatClient:
         if given_id in self._id_map:
             return self._id_map[given_id]
         elif given_id in self._buffer:
-            return {"fn": "get_result", "args": {"id": given_id}}
+            return {"fn": "get_result", "args": {"id": _wrap_arg(given_id)}}
         else:
             return given_id
 
@@ -253,9 +257,9 @@ class OatClient:
         fn_call = FunctionCall(
             fn="set_property",
             args={
-                "id": self._resolve_id(id),
-                "property": property,
-                "value": value
+                "id": _wrap_arg(self._resolve_id(id)),
+                "property": _wrap_arg(property),
+                "value": _wrap_arg(value)
             }
         )
         self._buffer[fn_call.hash()] = fn_call
@@ -266,8 +270,8 @@ class OatClient:
         self._buffer[id] = FunctionCall(
             fn="set_primitive",
             args={
-                "id": id,
-                "bound": [int(bound.real), int(bound.imag)]
+                "id": _wrap_arg(id),
+                "bound": _wrap_arg([int(bound.real), int(bound.imag)])
             }
         )
         if self.compilation_setting == CompilationSetting.INSTANT:
@@ -280,8 +284,8 @@ class OatClient:
             self._buffer[id] = FunctionCall(
                 fn="set_primitives",
                 args={
-                    "ids": ids,
-                    "bound": [int(bound.real), int(bound.imag)]
+                    "ids": _wrap_arg(ids),
+                    "bound": _wrap_arg([int(bound.real), int(bound.imag)])
                 }
             )
         if self.compilation_setting == CompilationSetting.INSTANT:
@@ -293,12 +297,15 @@ class OatClient:
         fn_data = FunctionCall(
             fn="set_gelineq",
             args={
-                "coefficients": [
-                    {"id": self._resolve_id(k), "coefficient": v}
+                "coefficients": _wrap_arg([
+                    {
+                        "id": _wrap_arg(self._resolve_id(k)),
+                        "coefficient": _wrap_arg(v)
+                    }
                     for k, v in coefficients.items()
-                ],
-                "bias": bias,
-                "alias": alias
+                ]),
+                "bias": _wrap_arg(bias),
+                "alias": _wrap_arg(alias) if alias is not None else _wrap_arg(None)
             },
         )
         fn_hash = fn_data.hash()
@@ -441,9 +448,12 @@ class OatClient:
                 "value": "red"
             })
         """
+        args = {}
+        if filter:
+            args["filter"] = _wrap_arg(filter)
         fn_data = FunctionCall(
             fn="get_node_ids",
-            args={"filter": filter} if filter else {}
+            args=args
         )
         result = self._execute_fn(fn_data)
         return result
@@ -452,7 +462,7 @@ class OatClient:
         fn_data = FunctionCall(
             fn="get_properties",
             args={
-                "id": self._resolve_id(id)
+                "id": _wrap_arg(self._resolve_id(id))
             }
         )
         result = self._execute_fn(fn_data)
@@ -462,7 +472,7 @@ class OatClient:
         fn_data = FunctionCall(
             fn="get_many_properties",
             args={
-                "ids": [self._resolve_id(id) for id in ids]
+                "ids": _wrap_arg([self._resolve_id(id) for id in ids])
             }
         )
         return self._execute_fn(fn_data)
@@ -478,7 +488,7 @@ class OatClient:
         """
         fn_data = FunctionCall(
             fn="get_alias",
-            args={"id": self._resolve_id(id)}
+            args={"id": _wrap_arg(self._resolve_id(id))}
         )
         return self._execute_fn(fn_data)
 
@@ -493,7 +503,7 @@ class OatClient:
         """
         fn_data = FunctionCall(
             fn="get_aliases_from_id",
-            args={"id": self._resolve_id(id)}
+            args={"id": _wrap_arg(self._resolve_id(id))}
         )
         return self._execute_fn(fn_data)
 
@@ -508,7 +518,7 @@ class OatClient:
         """
         fn_data = FunctionCall(
             fn="get_ids_from_aliases",
-            args={"aliases": aliases}
+            args={"aliases": _wrap_arg(aliases)}
         )
         return self._execute_fn(fn_data)
 
@@ -516,25 +526,28 @@ class OatClient:
         result = self._execute_fns([
             FunctionCall(
                 fn="sub",
-                args={"roots": [self._resolve_id(r) for r in roots]},
+                args={"roots": _wrap_arg([self._resolve_id(r) for r in roots])},
                 out="dag"
             ),
             FunctionCall(
                 fn="solve",
                 args={
-                    "dag": {
+                    "dag": _wrap_arg({
                         "fn": "get_result",
-                        "args": {"id": "dag"}
-                    },
-                    "objectives": [
+                        "args": {"id": _wrap_arg("dag")}
+                    }),
+                    "objectives": _wrap_arg([
                         {self._resolve_id(k): v for k, v in objective.items()}
                         for objective in objectives
-                    ],
-                    "assume": [
-                        {"id": self._resolve_id(k), "bound": [int(v.real), int(v.imag)]} 
+                    ]),
+                    "assume": _wrap_arg([
+                        {
+                            "id": _wrap_arg(self._resolve_id(k)),
+                            "bound": _wrap_arg([int(v.real), int(v.imag)])
+                        }
                         for k, v in assume.items()
-                    ],
-                    "maximize": maximize
+                    ]),
+                    "maximize": _wrap_arg(maximize)
                 },
                 out="solutions"
             )
@@ -551,10 +564,10 @@ class OatClient:
             FunctionCall(
                 fn="propagate",
                 args={
-                    "assignments": {
+                    "assignments": _wrap_arg({
                         self._resolve_id(k): [int(v.real), int(v.imag)]
                         for k, v in assignments.items()
-                    }
+                    })
                 }
             )
         )
@@ -564,16 +577,16 @@ class OatClient:
         fn_data = FunctionCall(
             fn="sub",
             args={
-                "roots": [self._resolve_id(r) for r in roots]
+                "roots": _wrap_arg([self._resolve_id(r) for r in roots])
             }
         )
         return self._execute_fn(fn_data)
-    
+
     def delete_node(self, id: str) -> None:
         fn_data = FunctionCall(
             fn="delete_node",
             args={
-                "id": self._resolve_id(id)
+                "id": _wrap_arg(self._resolve_id(id))
             }
         )
         self._execute_fn(fn_data)
